@@ -13,7 +13,6 @@ import inspect
 # Local imports
 from .base_data import BaseData, Indexer, FauxTrial
 from .methods.definetrial import definetrial
-from .methods.selectdata import selectdata
 from syncopy.shared.parsers import scalar_parser, array_parser
 from syncopy.shared.errors import SPYValueError
 from syncopy.shared.tools import best_match
@@ -29,7 +28,7 @@ class DiscreteData(BaseData, ABC):
     This class cannot be instantiated. Use one of the children instead.
     """
 
-    _infoFileProperties = BaseData._infoFileProperties + ("_hdr", "samplerate", )
+    _infoFileProperties = BaseData._infoFileProperties + ("samplerate", )
     _hdfFileAttributeProperties = BaseData._hdfFileAttributeProperties + ("samplerate",)
     _hdfFileDatasetProperties = BaseData._hdfFileDatasetProperties + ("data",)
 
@@ -59,7 +58,7 @@ class DiscreteData(BaseData, ABC):
     def __str__(self):
         # Get list of print-worthy attributes
         ppattrs = [attr for attr in self.__dir__()
-                   if not (attr.startswith("_") or attr in ["log", "trialdefinition", "hdr"])]
+                   if not (attr.startswith("_") or attr in ["log", "trialdefinition"])]
         ppattrs = [attr for attr in ppattrs
                    if not (inspect.ismethod(getattr(self, attr))
                            or isinstance(getattr(self, attr), Iterator))]
@@ -113,14 +112,6 @@ class DiscreteData(BaseData, ABC):
         ppstr += "\nUse `.log` to see object history"
         return ppstr
 
-    @property
-    def hdr(self):
-        """dict with information about raw data
-
-        This property is empty for data created by Syncopy.
-        """
-        return self._hdr
-
     @property
     def sample(self):
         """Indices of all recorded samples"""
@@ -145,6 +136,13 @@ class DiscreteData(BaseData, ABC):
             raise exc
         self._samplerate = sr
 
+    @property
+    def time(self):
+        """list(float): trigger-relative time of each event """
+        if self.samplerate is not None and self.sampleinfo is not None:
+            return [(trl[:,self.dimord.index("sample")] - self.sampleinfo[tk,0] + self._t0[tk]) / self.samplerate \
+                    for tk, trl in enumerate(self.trials)]
+
     @property
     def trialid(self):
         """:class:`numpy.ndarray` of trial id associated with the sample"""
@@ -182,9 +180,9 @@ class DiscreteData(BaseData, ABC):
     def trialtime(self):
         """list(:class:`numpy.ndarray`): trigger-relative sample times in s"""
         if self.samplerate is not None and self.sampleinfo is not None:
-            return [np.array([(t + self._t0[tk]) / self.samplerate \
-                              for t in range(0, int(self.sampleinfo[tk, 1] - self.sampleinfo[tk, 0]))]) \
-                    for tk in np.unique(self.trialid)]
+            sample0 = self.sampleinfo[:,0] - self._t0[:]
+            sample0 = np.append(sample0, np.nan)[self.trialid]
+            return (self.data[:,self.dimord.index("sample")] - sample0)/self.samplerate
 
     # Helper function that grabs a single trial
     def _get_trial(self, trialno):
@@ -266,41 +264,31 @@ class DiscreteData(BaseData, ABC):
         if toilim is not None:
             allTrials = self.trialtime
             for trlno in trials:
-                thisTrial = self.data[self.trialid == trlno, self.dimord.index("sample")]
-                trlSample = np.arange(*self.sampleinfo[trlno, :])
-                trlTime = allTrials[trlno]
-                minSample = trlSample[np.where(trlTime >= toilim[0])[0][0]]
-                maxSample = trlSample[np.where(trlTime <= toilim[1])[0][-1]]
-                selSample, _ = best_match(trlSample, [minSample, maxSample], span=True)
-                idxList = []
-                for smp in selSample:
-                    idxList += list(np.where(thisTrial == smp)[0])
-                if len(idxList) > 1:
-                    sampSteps = np.diff(idxList)
-                    if sampSteps.min() == sampSteps.max() == 1:
-                        idxList = slice(idxList[0], idxList[-1] + 1, 1)
-                timing.append(idxList)
+                trlTime = allTrials[self.trialid == trlno]
+                _, selTime = best_match(trlTime, toilim, span=True)
+                selTime = selTime.tolist()
+                if len(selTime) > 1 and np.diff(trlTime).min() > 0:
+                    timing.append(slice(selTime[0], selTime[-1] + 1, 1))
+                else:
+                    timing.append(selTime)
 
         elif toi is not None:
             allTrials = self.trialtime
             for trlno in trials:
-                thisTrial = self.data[self.trialid == trlno, self.dimord.index("sample")]
-                trlSample = np.arange(*self.sampleinfo[trlno, :])
-                trlTime = allTrials[trlno]
-                _, selSample = best_match(trlTime, toi)
-                for k, idx in enumerate(selSample):
-                    if np.abs(trlTime[idx - 1] - toi[k]) < np.abs(trlTime[idx] - toi[k]):
-                        selSample[k] = trlSample[idx -1]
-                    else:
-                        selSample[k] = trlSample[idx]
-                idxList = []
-                for smp in selSample:
-                    idxList += list(np.where(thisTrial == smp)[0])
-                if len(idxList) > 1:
-                    sampSteps = np.diff(idxList)
-                    if sampSteps.min() == sampSteps.max() == 1:
-                        idxList = slice(idxList[0], idxList[-1] + 1, 1)
-                timing.append(idxList)
+                trlTime = allTrials[self.trialid == trlno]
+                _, arrayIdx = best_match(trlTime, toi)
+                # squash duplicate values then readd
+                _, xdi = np.unique(trlTime[arrayIdx], return_index=True)
+                arrayIdx = arrayIdx[np.sort(xdi)]
+                selTime = []
+                for t in arrayIdx:
+                    selTime += np.where(trlTime[t] == trlTime)[0].tolist()
+                # convert to slice if possible
+                if len(selTime) > 1:
+                    timeSteps = np.diff(selTime)
+                    if timeSteps.min() == timeSteps.max() == 1:
+                        selTime = slice(selTime[0], selTime[-1] + 1, 1)
+                timing.append(selTime)
 
         else:
             timing = [slice(None)] * len(trials)
@@ -312,7 +300,6 @@ class DiscreteData(BaseData, ABC):
         # Assign (default) values
         self._trialid = None
         self._samplerate = None
-        self._hdr = None
         self._data = None
 
         self.samplerate = samplerate
@@ -321,7 +308,7 @@ class DiscreteData(BaseData, ABC):
         # Call initializer
         super().__init__(data=data, **kwargs)
 
-        if self.data is not None:
+        if self.data is not None and self.data.size != 0:
 
             # In case of manual data allocation (reading routine would leave a
             # mark in `cfg`), fill in missing info
@@ -338,9 +325,7 @@ class SpikeData(DiscreteData):
     stored as a two-dimensional [nSpikes x 3] array on disk with the columns
     being ``["sample", "channel", "unit"]``.
 
-    Data is only read from disk on demand, similar to memory maps and HDF5
-    files.
-
+    Data is only read from disk on demand, similar to HDF5 files.
     """
 
     _infoFileProperties = DiscreteData._infoFileProperties + ("channel", "unit",)
@@ -488,8 +473,7 @@ class SpikeData(DiscreteData):
                 ordered list of dimension labels
 
         1. `filename` + `data` : create hdf dataset incl. sampleinfo @filename
-        2. `filename` no `data` : read from file or memmap (spy, hdf5, npy file
-           array -> memmap)
+        2. `filename` no `data` : read from file (spy, hdf5 file)
         3. just `data` : try to attach data (error checking done by
            :meth:`SpikeData.data.setter`)
 
@@ -520,9 +504,7 @@ class EventData(DiscreteData):
     stimulus was turned on, etc. These usually occur at non-regular time points
     and have associated event codes.
 
-    Data is only read from disk on demand, similar to memory maps and HDF5
-    files.
-
+    Data is only read from disk on demand, similar to HDF5 files.
     """
 
     _defaultDimord = ["sample", "eventid"]
@@ -607,8 +589,7 @@ class EventData(DiscreteData):
                 ordered list of dimension labels
 
         1. `filename` + `data` : create hdf dataset incl. sampleinfo @filename
-        2. `filename` no `data` : read from file or memmap (spy, hdf5, npy file
-           array -> memmap)
+        2. `filename` no `data` : read from file(spy, hdf5)
         3. just `data` : try to attach data (error checking done by
            :meth:`EventData.data.setter`)
 
@@ -617,6 +598,15 @@ class EventData(DiscreteData):
         :func:`syncopy.definetrial`
 
         """
+        if dimord is not None:
+            # ensure that event data can have extra dimord columns
+            if len(dimord) != len(self._defaultDimord):
+                for col in self._defaultDimord:
+                    if col not in dimord:
+                        base = "dimensional label {}"
+                        lgl = base.format("'" + col + "'")
+                        raise SPYValueError(legal=lgl, varname="dimord")
+                self._defaultDimord = dimord
 
         # Call parent initializer
         super().__init__(data=data,
