@@ -9,12 +9,14 @@ from . import (
     load_spy_container,
     save_spy_container,
     load_ft,
+    load_tdt,
     _load_nwb
 )
 from .utils import *
 from .load_spy_container import *
 from .save_spy_container import *
 from .load_ft import *
+from .load_tdt import *
 from ._load_nwb import *
 
 # Populate local __all__ namespace
