@@ -20,6 +20,9 @@ from syncopy.io.utils import hash_file, startInfoDict
 
 import syncopy.datatype as spd
 
+# to allow loading older spy containers
+legacy_not_required = ['info']
+
 __all__ = ["load"]
 
 
@@ -252,18 +255,12 @@ def _load(filename, checksum, mode, out):
     requiredFields = tuple(startInfoDict.keys()) + dataclass._infoFileProperties
 
     for key in requiredFields:
-        if key not in jsonDict.keys():
+        if key not in jsonDict.keys() and key not in legacy_not_required:
             raise SPYError("Required field {field} for {cls} not in {file}"
                            .format(field=key,
                                    cls=dataclass.__name__,
                                    file=jsonFile))
 
-    # If `_hdr` is an empty list, set it to `None` to not confuse meta-functions
-    hdr = jsonDict.get("_hdr")
-    if isinstance(hdr, (list, np.ndarray)):
-        if len(hdr) == 0:
-            jsonDict["_hdr"] = None
-
     # FIXME: add version comparison (syncopy.__version__ vs jsonDict["_version"])
 
     # If wanted, perform checksum matching
@@ -276,20 +273,18 @@ def _load(filename, checksum, mode, out):
                                 actual=hsh_msg.format(hsh=hsh))
 
     # Parsing is done, create new or check provided object
+    dimord = jsonDict.pop("dimord")
     if out is not None:
         try:
             data_parser(out, varname="out", writable=True, dataclass=jsonDict["dataclass"])
         except Exception as exc:
             raise exc
         new_out = False
+        out.dimord = dimord
     else:
-        out = dataclass()
+        out = dataclass(dimord=dimord)
         new_out = True
 
-    # First and foremost, assign dimensional information
-    dimord = jsonDict.pop("dimord")
-    out.dimord = dimord
-
     # Access data on disk (error checking is done by setters)
     out.mode = mode
     for datasetProperty in out._hdfFileDatasetProperties:
@@ -301,13 +296,11 @@ def _load(filename, checksum, mode, out):
     out.definetrial(trialdef)
 
     # Assign metadata
-    for key in [prop for prop in dataclass._infoFileProperties if prop != "dimord"]:
+    for key in [prop for prop in dataclass._infoFileProperties if
+                prop != "dimord" and prop in jsonDict.keys()]:
         setattr(out, key, jsonDict[key])
 
-    # Write `cfg` entries
     thisMethod = sys._getframe().f_code.co_name.replace("_", "")
-    out.cfg = {"method": thisMethod,
-               "files": [hdfFile, jsonFile]}
 
     # Write log-entry
     msg = "Read files v. {ver:s} ".format(ver=jsonDict["_version"])
