@@ -10,14 +10,16 @@ import scipy.signal as sci
 from inspect import signature
 
 # syncopy imports
-from syncopy.shared.computational_routine import ComputationalRoutine
-from syncopy.shared.kwarg_decorators import unwrap_io
+from syncopy.shared.computational_routine import ComputationalRoutine, propagate_metadata
+from syncopy.shared.const_def import spectralConversions, spectralDTypes
+from syncopy.shared.kwarg_decorators import process_io
 
 # backend imports
 from .firws import design_wsinc, apply_fir, minphaserceps
+from .resampling import downsample, resample
 
 
-@unwrap_io
+@process_io
 def sinc_filtering_cF(dat,
                       samplerate=1,
                       filter_type='lp',
@@ -131,7 +133,7 @@ def sinc_filtering_cF(dat,
     return filtered
 
 
-class Sinc_Filtering(ComputationalRoutine):
+class SincFiltering(ComputationalRoutine):
 
     """
     Compute class that performs filtering with windowed sinc filters
@@ -153,21 +155,10 @@ class Sinc_Filtering(ComputationalRoutine):
 
     def process_metadata(self, data, out):
 
-        # Some index gymnastics to get trial begin/end "samples"
-        if data.selection is not None:
-            chanSec = data.selection.channel
-            trl = data.selection.trialdefinition
-        else:
-            chanSec = slice(None)
-            trl = data.trialdefinition
-
-        out.trialdefinition = trl
-
-        out.samplerate = data.samplerate
-        out.channel = np.array(data.channel[chanSec])
+        propagate_metadata(data, out)
 
 
-@unwrap_io
+@process_io
 def but_filtering_cF(dat,
                      samplerate=1,
                      filter_type='lp',
@@ -265,7 +256,7 @@ def but_filtering_cF(dat,
         return filtered
 
 
-class But_Filtering(ComputationalRoutine):
+class ButFiltering(ComputationalRoutine):
 
     """
     Compute class that performs filtering with butterworth filters
@@ -287,21 +278,10 @@ class But_Filtering(ComputationalRoutine):
 
     def process_metadata(self, data, out):
 
-        # Some index gymnastics to get trial begin/end "samples"
-        if data.selection is not None:
-            chanSec = data.selection.channel
-            trl = data.selection.trialdefinition
-        else:
-            chanSec = slice(None)
-            trl = data.trialdefinition
-
-        out.trialdefinition = trl
-
-        out.samplerate = data.samplerate
-        out.channel = np.array(data.channel[chanSec])
+        propagate_metadata(data, out)
 
 
-@unwrap_io
+@process_io
 def rectify_cF(dat, noCompute=False, chunkShape=None):
 
     """
@@ -360,21 +340,10 @@ class Rectify(ComputationalRoutine):
 
     def process_metadata(self, data, out):
 
-        # Some index gymnastics to get trial begin/end "samples"
-        if data.selection is not None:
-            chanSec = data.selection.channel
-            trl = data.selection.trialdefinition
-        else:
-            chanSec = slice(None)
-            trl = data.trialdefinition
-
-        out.trialdefinition = trl
-
-        out.samplerate = data.samplerate
-        out.channel = np.array(data.channel[chanSec])
+        propagate_metadata(data, out)
 
 
-@unwrap_io
+@process_io
 def hilbert_cF(dat, output='abs', timeAxis=0, noCompute=False, chunkShape=None):
 
     """
@@ -409,16 +378,6 @@ def hilbert_cF(dat, output='abs', timeAxis=0, noCompute=False, chunkShape=None):
 
     """
 
-    out_trafo = {
-        'abs': lambda x: np.abs(x),
-        'complex': lambda x: x,
-        'real': lambda x: np.real(x),
-        'imag': lambda x: np.imag(x),
-        'absreal': lambda x: np.abs(np.real(x)),
-        'absimag': lambda x: np.abs(np.imag(x)),
-        'angle': lambda x: np.angle(x)
-    }
-
     # Re-arrange array if necessary and get dimensional information
     if timeAxis != 0:
         dat = dat.T       # does not copy but creates view of `dat`
@@ -428,13 +387,13 @@ def hilbert_cF(dat, output='abs', timeAxis=0, noCompute=False, chunkShape=None):
     # operation does not change the shape
     # but may change the number format
     outShape = dat.shape
-    fmt = np.complex64 if output == 'complex' else np.float32
+    fmt = spectralDTypes["fourier"] if output == 'complex' else spectralDTypes["abs"]
     if noCompute:
         return outShape, fmt
 
     trafo = sci.hilbert(dat, axis=0)
 
-    return out_trafo[output](trafo)
+    return spectralConversions[output](trafo)
 
 
 class Hilbert(ComputationalRoutine):
@@ -459,22 +418,10 @@ class Hilbert(ComputationalRoutine):
 
     def process_metadata(self, data, out):
 
-        # Some index gymnastics to get trial begin/end "samples"
-        if data.selection is not None:
-            chanSec = data.selection.channel
-            trl = data.selection.trialdefinition
-
-        else:
-            chanSec = slice(None)
-            trl = data.trialdefinition
-
-        out.trialdefinition = trl
-
-        out.samplerate = data.samplerate
-        out.channel = np.array(data.channel[chanSec])
+        propagate_metadata(data, out)
 
 
-@unwrap_io
+@process_io
 def downsample_cF(dat,
                   samplerate=1,
                   new_samplerate=1,
@@ -483,7 +430,7 @@ def downsample_cF(dat,
                   noCompute=False
                   ):
     """
-    Provides basic downsampling of signals. The `new_samplerate` should be 
+    Provides basic downsampling of signals. The `new_samplerate` should be
     an integer division of the original `samplerate`.
 
     dat : (N, K) :class:`numpy.ndarray`
@@ -499,7 +446,7 @@ def downsample_cF(dat,
 
     Returns
     -------
-    filtered : (X, K) :class:`~numpy.ndarray`
+    resampled : (X, K) :class:`~numpy.ndarray`
         The downsampled data
 
     Notes
@@ -519,23 +466,23 @@ def downsample_cF(dat,
     else:
         dat = dat
 
-    # we need integers for slicing
-    skipped = int(samplerate // new_samplerate)
-
-    outShape = list(dat.shape)
-    outShape[0] = int(np.ceil(dat.shape[0] / skipped))
-
     if noCompute:
+        # we need integers for slicing
+        skipped = int(samplerate // new_samplerate)
+        outShape = list(dat.shape)
+        outShape[0] = int(np.ceil(dat.shape[0] / skipped))
         return tuple(outShape), dat.dtype
 
-    return dat[::skipped]
+    resampled = downsample(dat, samplerate, new_samplerate)
+
+    return resampled
 
 
 class Downsample(ComputationalRoutine):
 
     """
-    Compute class that performs straightforward downsampling
-    of :class:`~syncopy.AnalogData` objects
+    Compute class that performs straightforward (integer division)
+    downsampling of :class:`~syncopy.AnalogData` objects
 
     Sub-class of :class:`~syncopy.shared.computational_routine.ComputationalRoutine`,
     see :doc:`/developer/compute_kernels` for technical details on Syncopy's compute
@@ -555,10 +502,7 @@ class Downsample(ComputationalRoutine):
 
         # we need to re-calculate the downsampling factor
         factor = int(data.samplerate // self.cfg['new_samplerate'])
-        
-        # now set new samplerate
-        data.samplerate = self.cfg['new_samplerate']
-        
+
         if data.selection is not None:
             chanSec = data.selection.channel
             trl = data.selection.trialdefinition // factor
@@ -567,6 +511,304 @@ class Downsample(ComputationalRoutine):
             trl = data.trialdefinition // factor
 
         out.trialdefinition = trl
-
-        out.samplerate = data.samplerate
+        # now set new samplerate
+        out.samplerate = self.cfg['new_samplerate']
         out.channel = np.array(data.channel[chanSec])
+
+
+@process_io
+def resample_cF(dat,
+                samplerate=1,
+                new_samplerate=1,
+                lpfreq=None,
+                order=None,
+                timeAxis=0,
+                chunkShape=None,
+                noCompute=False
+                ):
+    """
+    Provides resampling of signals. The `new_samplerate` can be
+    any (rational) factor of the original `samplerate`.
+
+    For the anti-aliasing an explicit low-pass firws filter
+    is constructed. Either implicitly with `lpfreq=None`
+    which takes the new Nyquist (new_samplerate / 2) as cut-off
+    or with an explicit `lpfreq` as the cut-off frequency.
+
+    dat : (N, K) :class:`numpy.ndarray`
+        Uniformly sampled multi-channel time-series data
+        The 1st dimension is interpreted as the time axis,
+        columns represent individual channels.
+    samplerate : float
+        Sample rate of the input data in Hz
+    new_samplerate : float
+        Sample rate of the output data in Hz
+    lpfreq : None or float, optional
+        Leave at `None` for standard anti-alias filtering with
+        the new Nyquist or set explicitly in Hz
+    order : None or int, optional
+        Order (length) of the firws anti-aliasing filter.
+        The default `None` will create a filter of
+        maximal order which is the number of samples in the trial.
+    timeAxis : int, optional
+        Index of running time axis in `dat` (0 or 1)
+
+    Returns
+    -------
+    resampled : (X, K) :class:`~numpy.ndarray`
+        The resampled data
+
+    Notes
+    -----
+    This method is intended to be used as
+    :meth:`~syncopy.shared.computational_routine.ComputationalRoutine.computeFunction`
+    inside a :class:`~syncopy.shared.computational_routine.ComputationalRoutine`.
+    Thus, input parameters are presumed to be forwarded from a parent metafunction.
+    Consequently, this function does **not** perform any error checking and operates
+    under the assumption that all inputs have been externally validated and cross-checked.
+
+    """
+
+    # Re-arrange array if necessary and get dimensional information
+    if timeAxis != 0:
+        dat = dat.T       # does not copy but creates view of `dat`
+    else:
+        dat = dat
+
+    nSamples = dat.shape[0]
+    fs_ratio = new_samplerate / samplerate
+
+    if noCompute:
+        new_nSamples = int(np.ceil(nSamples * fs_ratio))
+        return (new_nSamples, dat.shape[1]), dat.dtype
+
+    resampled = resample(dat,
+                         samplerate,
+                         new_samplerate,
+                         lpfreq=lpfreq,
+                         order=order)
+
+    return resampled
+
+
+class Resample(ComputationalRoutine):
+
+    """
+    Compute class that performs resampling (up-fir-down)
+    of :class:`~syncopy.AnalogData` objects
+
+    Sub-class of :class:`~syncopy.shared.computational_routine.ComputationalRoutine`,
+    see :doc:`/developer/compute_kernels` for technical details on Syncopy's compute
+    classes and metafunctions.
+
+    See also
+    --------
+    syncopy.preprocessing : parent metafunction
+    """
+
+    computeFunction = staticmethod(resample_cF)
+
+    # 1st argument,the data, gets omitted
+    valid_kws = list(signature(downsample_cF).parameters.keys())[1:]
+
+    def process_metadata(self, data, out):
+
+        # we need to re-calculate the resampling factor
+        factor = self.cfg['new_samplerate'] / data.samplerate
+        trafo_trl = lambda trldef: np.ceil(trldef * factor)
+
+        if data.selection is not None:
+            chanSec = data.selection.channel
+            trl = trafo_trl(data.selection.trialdefinition)
+        else:
+            chanSec = slice(None)
+            trl = trafo_trl(data.trialdefinition)
+
+        out.trialdefinition = trl
+
+        # now set new samplerate
+        out.samplerate = self.cfg['new_samplerate']
+        out.channel = np.array(data.channel[chanSec])
+
+
+@process_io
+def detrending_cF(dat, polyremoval=None, timeAxis=0, noCompute=False, chunkShape=None):
+
+    """
+    Simple cF to wire SciPy's `detrend` to our CRs,
+    supported are constant and linear detrending
+
+    Parameters
+    ----------
+    dat : (N, K) :class:`numpy.ndarray`
+        Uniformly sampled multi-channel time-series data
+        The 1st dimension is interpreted as the time axis,
+        columns represent individual channels.
+        Dimensions can be transposed to `(K, N)` with the `timeAxis` parameter
+    polyremoval : {0, 1} or None
+        Order of polynomial used for de-trending data in the time domain.
+        A value of 0 corresponds to subtracting the mean
+        ("de-meaning"), ``polyremoval = 1`` removes linear trends (subtracting the
+        least squares fit of a linear polynomial).
+    timeAxis : int, optional
+        Index of running time axis in `dat` (0 or 1)
+    noCompute : bool
+        Preprocessing flag. If `True`, do not perform actual calculation but
+        instead return expected shape and :class:`numpy.dtype` of output
+        array.
+
+    Returns
+    -------
+    detrended : (N, K) :class:`~numpy.ndarray`
+        The detrended signals
+
+    Notes
+    -----
+    This method is intended to be used as
+    :meth:`~syncopy.shared.computational_routine.ComputationalRoutine.computeFunction`
+    inside a :class:`~syncopy.shared.computational_routine.ComputationalRoutine`.
+    Thus, input parameters are presumed to be forwarded from a parent metafunction.
+    Consequently, this function does **not** perform any error checking and operates
+    under the assumption that all inputs have been externally validated and cross-checked.
+    """
+
+    # should be captured in frontend
+    if polyremoval is None:
+        return dat
+
+    # Re-arrange array if necessary and get dimensional information
+    if timeAxis != 0:
+        dat = dat.T       # does not copy but creates view of `dat`
+    else:
+        dat = dat
+
+    # detrending does not change the shape
+    outShape = dat.shape
+    if noCompute:
+        return outShape, np.float32
+
+    # detrend
+    if polyremoval == 0:
+        dat = sci.detrend(dat, type='constant', axis=0, overwrite_data=True)
+    elif polyremoval == 1:
+        dat = sci.detrend(dat, type='linear', axis=0, overwrite_data=True)
+
+    # renaming
+    detrended = dat
+    return detrended
+
+
+class Detrending(ComputationalRoutine):
+
+    """
+    Compute class that performs constant or linear detrending
+    of :class:`~syncopy.AnalogData` objects
+
+    Sub-class of :class:`~syncopy.shared.computational_routine.ComputationalRoutine`,
+    see :doc:`/developer/compute_kernels` for technical details on Syncopy's compute
+    classes and metafunctions.
+
+    See also
+    --------
+    syncopy.preprocessing : parent metafunction
+    """
+
+    computeFunction = staticmethod(detrending_cF)
+
+    # 1st argument,the data, gets omitted
+    valid_kws = list(signature(detrending_cF).parameters.keys())[1:]
+
+    def process_metadata(self, data, out):
+
+        propagate_metadata(data, out)
+
+
+@process_io
+def standardize_cF(dat, polyremoval=None, timeAxis=0, noCompute=False, chunkShape=None):
+
+    """
+    Yet another simple cF to z-score ('standardize') signals:
+    subtracting the mean and normalize by standard deviation.
+
+    Parameters
+    ----------
+    dat : (N, K) :class:`numpy.ndarray`
+        Uniformly sampled multi-channel time-series data
+        The 1st dimension is interpreted as the time axis,
+        columns represent individual channels.
+        Dimensions can be transposed to `(K, N)` with the `timeAxis` parameter
+    polyremoval : {0, 1} or None
+        Order of polynomial used for de-trending data in the time domain prior
+        to z-scoring. A value of 0 corresponds to subtracting the mean
+        ("de-meaning"), ``polyremoval = 1`` removes linear trends (subtracting the
+        least squares fit of a linear polynomial).
+    timeAxis : int, optional
+        Index of running time axis in `dat` (0 or 1)
+    noCompute : bool
+        Preprocessing flag. If `True`, do not perform actual calculation but
+        instead return expected shape and :class:`numpy.dtype` of output
+        array.
+
+    Returns
+    -------
+    standardized : (N, K) :class:`~numpy.ndarray`
+        The standardized signals
+
+    Notes
+    -----
+    This method is intended to be used as
+    :meth:`~syncopy.shared.computational_routine.ComputationalRoutine.computeFunction`
+    inside a :class:`~syncopy.shared.computational_routine.ComputationalRoutine`.
+    Thus, input parameters are presumed to be forwarded from a parent metafunction.
+    Consequently, this function does **not** perform any error checking and operates
+    under the assumption that all inputs have been externally validated and cross-checked.
+    """
+
+    # Re-arrange array if necessary and get dimensional information
+    if timeAxis != 0:
+        dat = dat.T       # does not copy but creates view of `dat`
+    else:
+        dat = dat
+
+    # cF does not change the shape
+    outShape = dat.shape
+    if noCompute:
+        return outShape, np.float32
+
+    # detrend
+    if polyremoval == 0:
+        dat = sci.detrend(dat, type='constant', axis=0, overwrite_data=True)
+    elif polyremoval == 1:
+        dat = sci.detrend(dat, type='linear', axis=0, overwrite_data=True)
+
+    # standardize
+    dat = (dat - np.mean(dat, axis=0)) / np.std(dat, axis=0)
+
+    # renaming
+    standardized = dat
+    return standardized
+
+
+class Standardize(ComputationalRoutine):
+
+    """
+    Compute class that performs standardizing
+    of :class:`~syncopy.AnalogData` objects
+
+    Sub-class of :class:`~syncopy.shared.computational_routine.ComputationalRoutine`,
+    see :doc:`/developer/compute_kernels` for technical details on Syncopy's compute
+    classes and metafunctions.
+
+    See also
+    --------
+    syncopy.preprocessing : parent metafunction
+    """
+
+    computeFunction = staticmethod(standardize_cF)
+
+    # 1st argument,the data, gets omitted
+    valid_kws = list(signature(standardize_cF).parameters.keys())[1:]
+
+    def process_metadata(self, data, out):
+
+        propagate_metadata(data, out)
