@@ -12,7 +12,6 @@ import numpy as np
 from itertools import chain
 from abc import ABC, abstractmethod
 from copy import copy
-from numpy.lib.format import open_memmap
 from tqdm.auto import tqdm
 if sys.platform == "win32":
     # tqdm breaks term colors on Windows - fix that (tqdm issue #446)
@@ -21,8 +20,9 @@ if sys.platform == "win32":
     colorama.init(strip=False)
 
 # Local imports
+import syncopy as spy
 from .tools import get_defaults
-from syncopy import __storage__, __acme__, __path__
+from syncopy import __storage__, __acme__
 from syncopy.shared.errors import SPYValueError, SPYTypeError, SPYParallelError, SPYWarning
 if __acme__:
     from acme import ParallelMap
@@ -139,9 +139,6 @@ class ComputationalRoutine(ABC):
         # numerical type of output dataset
         self.dtype = None
 
-        # list of dicts encoding header info of raw binary input files (experimental!)
-        self.hdr = None
-
         # list of trial numbers to process (either `data.trials` or `data.selection.trials`)
         self.trialList = None
 
@@ -285,8 +282,9 @@ class ComputationalRoutine(ABC):
         trials = []
         for tk, trialno in enumerate(self.trialList):
             trial = data._preview_trial(trialno)
-            trlArg = tuple(arg[tk] if isinstance(arg, (list, tuple, np.ndarray)) and len(arg) == self.numTrials \
-                else arg for arg in self.argv)
+            trlArg = tuple(arg[tk] if isinstance(arg, (list, tuple, np.ndarray)) and
+                           len(arg) == self.numTrials
+                           else arg for arg in self.argv)
             chunkShape, dtype = self.computeFunction(trial,
                                                      *trlArg,
                                                      **dryRunKwargs)
@@ -337,8 +335,9 @@ class ComputationalRoutine(ABC):
 
         # Allocate control variables
         trial = trials[0]
-        trlArg0 = tuple(arg[0] if isinstance(arg, (list, tuple, np.ndarray)) and len(arg) == self.numTrials \
-            else arg for arg in self.argv)
+        trlArg0 = tuple(arg[0] if isinstance(arg, (list, tuple, np.ndarray)) and
+                        len(arg) == self.numTrials
+                        else arg for arg in self.argv)
         chunkShape0 = tuple(chk_arr[0, :])
         lyt = [slice(0, stop) for stop in chunkShape0]
         sourceLayout = []
@@ -354,7 +353,7 @@ class ComputationalRoutine(ABC):
             # Set up channel-chunking: `c_blocks` holds channel blocks per trial
             nChannels = data.channel.size
             rem = int(nChannels % chan_per_worker)
-            c_blocks = [chan_per_worker] * int(nChannels//chan_per_worker) + [rem] * int(rem > 0)
+            c_blocks = [chan_per_worker] * int(nChannels // chan_per_worker) + [rem] * int(rem > 0)
             inchanidx = data.dimord.index("channel")
 
             # Perform dry-run w/first channel-block of first trial to identify
@@ -403,8 +402,9 @@ class ComputationalRoutine(ABC):
         stacking = targetLayout[0][stackingDim].stop
         for tk in range(1, self.numTrials):
             trial = trials[tk]
-            trlArg = tuple(arg[tk] if isinstance(arg, (list, tuple, np.ndarray)) and len(arg) == self.numTrials \
-                else arg for arg in self.argv)
+            trlArg = tuple(arg[tk] if isinstance(arg, (list, tuple, np.ndarray)) and
+                           len(arg) == self.numTrials
+                           else arg for arg in self.argv)
             chkshp = chk_list[tk]
             lyt = [slice(0, stop) for stop in chkshp]
             lyt[stackingDim] = slice(stacking, stacking + chkshp[stackingDim])
@@ -424,7 +424,7 @@ class ComputationalRoutine(ABC):
                     idx[inchanidx] = slice(blockstack, blockstack + block)
                     trial.shape = tuple(shp)
                     trial.idx = tuple(idx)
-                    res, _ = self.computeFunction(trial, *trlArg, **dryRunKwargs) # FauxTrial
+                    res, _ = self.computeFunction(trial, *trlArg, **dryRunKwargs)   # FauxTrial
                     lyt[outchanidx] = slice(chanstack, chanstack + res[outchanidx])
                     targetLayout.append(tuple(lyt))
                     targetShapes.append(tuple([slc.stop - slc.start for slc in lyt]))
@@ -462,7 +462,7 @@ class ComputationalRoutine(ABC):
                         sel = [sel]
                     if isinstance(sel, list):
                         selarr = np.array(sel, dtype=np.intp)
-                    else: # sel is a slice
+                    else:   # sel is a slice
                         step = sel.step
                         if sel.step is None:
                             step = 1
@@ -608,8 +608,7 @@ class ComputationalRoutine(ABC):
 
             # Construct list of dicts that will be passed on to workers: in the
             # parallel case, `trl_dat` is a dictionary!
-            workerDicts = [{"hdr": self.hdr,
-                            "keeptrials": self.keeptrials,
+            workerDicts = [{"keeptrials": self.keeptrials,
                             "infile": data.filename,
                             "indset": data.data.name,
                             "ingrid": self.sourceLayout[chk],
@@ -721,9 +720,6 @@ class ComputationalRoutine(ABC):
         # reading access to backing device on disk
         data.mode = "r"
 
-        # Take care of `VirtualData` objects
-        self.hdr = getattr(data, "hdr", None)
-
         # Perform actual computation
         computeMethod(data, out)
 
@@ -810,7 +806,7 @@ class ComputationalRoutine(ABC):
         Notes
         -----
         The actual reading of source data and writing of results is managed by
-        the decorator :func:`syncopy.shared.parsers.unwrap_io`.
+        the decorator :func:`syncopy.shared.kwarg_decorators.process_io`.
 
         See also
         --------
@@ -865,16 +861,7 @@ class ComputationalRoutine(ABC):
         compute_parallel : concurrent processing counterpart of this method
         """
 
-        # Initialize on-disk backing device (either HDF5 file or memmap)
-        if self.hdr is None:
-            try:
-                sourceObj = h5py.File(data.filename, mode="r")[data.data.name]
-                isHDF = True
-            except OSError:
-                sourceObj = open_memmap(data.filename, mode="c")
-                isHDF = False
-            except Exception as exc:
-                raise exc
+        sourceObj = h5py.File(data.filename, mode="r")[data.data.name]
 
         # Iterate over (selected) trials and write directly to target HDF5 dataset
         with h5py.File(out.filename, "r+") as h5fout:
@@ -886,9 +873,10 @@ class ComputationalRoutine(ABC):
                 ingrid = self.sourceLayout[nblock]
                 sigrid = self.sourceSelectors[nblock]
                 outgrid = self.targetLayout[nblock]
-                argv = tuple(arg[nblock] \
-                    if isinstance(arg, (list, tuple, np.ndarray)) and len(arg) == self.numTrials \
-                        else arg for arg in self.argv)
+                argv = tuple(arg[nblock]
+                             if isinstance(arg, (list, tuple, np.ndarray)) and
+                             len(arg) == self.numTrials
+                             else arg for arg in self.argv)
 
                 # Catch empty source-array selections; this workaround is not
                 # necessary for h5py version 2.10+ (see https://github.com/h5py/h5py/pull/1174)
@@ -896,28 +884,11 @@ class ComputationalRoutine(ABC):
                     res = np.empty(self.targetShapes[nblock], dtype=self.dtype)
                 else:
                     # Get source data as NumPy array
-                    if self.hdr is None:
-                        if isHDF:
-                            if self.useFancyIdx:
-                                arr = np.array(sourceObj[tuple(ingrid)])[np.ix_(*sigrid)]
-                            else:
-                                arr = np.array(sourceObj[tuple(ingrid)])
-                        else:
-                            if self.useFancyIdx:
-                                arr = sourceObj[np.ix_(*ingrid)]
-                            else:
-                                arr = np.array(sourceObj[ingrid])
-                        sourceObj.flush()
+                    if self.useFancyIdx:
+                        arr = np.array(sourceObj[tuple(ingrid)])[np.ix_(*sigrid)]
                     else:
-                        idx = ingrid
-                        if self.useFancyIdx:
-                            idx = np.ix_(*ingrid)
-                        stacks = []
-                        for fk, fname in enumerate(data.filename):
-                            stacks.append(np.memmap(fname, offset=int(self.hdr[fk]["length"]),
-                                                    mode="r", dtype=self.hdr[fk]["dtype"],
-                                                    shape=(self.hdr[fk]["M"], self.hdr[fk]["N"]))[idx])
-                        arr = np.vstack(stacks)[ingrid]
+                        arr = np.array(sourceObj[tuple(ingrid)])
+                    sourceObj.flush()
 
                     # Ensure input array shape was not inflated by scalar selection
                     # tuple, e.g., ``e=np.ones((2,2)); e[0,:].shape = (2,)`` not ``(1,2)``
@@ -946,8 +917,7 @@ class ComputationalRoutine(ABC):
                 target[()] /= self.numTrials
 
         # If source was HDF5 file, close it to prevent access errors
-        if isHDF:
-            sourceObj.file.close()
+        sourceObj.file.close()
 
         return
 
@@ -994,7 +964,6 @@ class ComputationalRoutine(ABC):
                                                         value=str(v) if len(str(v)) < 80
                                                         else str(v)[:30] + ", ..., " + str(v)[-30:])
         out.log = logHead + logOpts
-        out.cfg = cfg
 
     @abstractmethod
     def process_metadata(self, data, out):
@@ -1023,3 +992,63 @@ class ComputationalRoutine(ABC):
         write_log : Logging of calculation parameters
         """
         pass
+
+
+# --- metadata helper functions ---
+
+def propagate_metadata(in_data, out_data):
+
+    """
+    Propagating metadata/selections (channels, trials, time, ...)
+    from the input object `in_data` to the output
+    object `out_data` is the task of
+    concrete (overloaded) `process_metadata` implementations.
+
+    Depending on the concrete CR, different propagations are needed.
+    For example
+
+        AnalogData -> CrossSpectralData (connectivityanalysis)
+
+    needs a mapping .channel -> .channel_i, .channel_j
+
+    Whereas
+
+        AnalogData -> AnalogData (preprocessing)
+
+    directly propagates all (including selections)
+    attributes from the input to the output
+    (same channels, time and so on)
+
+    This function is a general approach to unify the
+    propagation / adaptation of all needed attributes.
+
+    Parameters
+    ----------
+    in_data : Syncopy data object
+        Input object to get attributes from
+    out_data : Syncopy data object
+        Output object to set attributes
+    """
+
+    # instance checkers
+    is_AD = lambda data: isinstance(data, spy.AnalogData)
+
+    # simplest case, direct propagation between two AnalogData objects
+    if is_AD(in_data) and is_AD(out_data):
+
+        # get channels and trial selections
+        if in_data.selection is not None:
+            chanSec = in_data.selection.channel
+            # captures toi selections
+            trl = in_data.selection.trialdefinition
+        else:
+            chanSec = slice(None)
+            trl = in_data.trialdefinition
+
+        out_data.trialdefinition = trl
+        out_data.channel = np.array(in_data.channel[chanSec])
+        out_data.samplerate = in_data.samplerate
+
+    # nothing else supported atm
+    else:
+        raise NotImplementedError
