@@ -6,6 +6,7 @@
 # Builtin/3rd party package imports
 import numpy as np
 import inspect
+import json
 
 # Local imports
 from syncopy.shared.errors import SPYValueError, SPYWarning, SPYTypeError
@@ -47,6 +48,75 @@ class StructDict(dict):
         return ppStr
 
 
+class SerializableDict(dict):
+
+    """
+    It's a dict which checks newly inserted
+    values for serializability, keys should always be serializable
+    """
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        # check also initial entries
+        for key, value in self.items():
+            self.is_json(key, value)
+
+    def __setitem__(self, key, value):
+        self.is_json(key, value)
+        dict.__setitem__(self, key, value)
+
+    def is_json(self, key, value):
+        try:
+            json.dumps(value)
+        except TypeError:
+            lgl = "serializable data type, e.g. floats, lists, tuples, ... "
+            raise SPYTypeError(value, f"value for key '{key}'", lgl)
+        try:
+            json.dumps(key)
+        except TypeError:
+            lgl = "serializable data type, e.g. floats, lists, tuples, ... "
+            raise SPYTypeError(value, f"key '{key}'", lgl)
+
+
+def get_frontend_cfg(defaults, lcls, kwargs):
+
+    """
+    Assemble cfg dict to allow direct replay of frontend calls
+
+    Parameters
+    ----------
+
+    defaults : dict
+        The result of :func:`~get_defaults`, holding all frontend specific
+        parameter names and default values
+    lcls : dict
+        The `locals()` within a frontend call, contains passed
+        parameter names and values
+    kwargs : dict
+        The `kwargs` attached to every frontend signature, holding
+        additional arguments, e.g. `parallel` and `select`
+
+    Returns
+    -------
+    new_cfg : :class:`~StructDict`
+        Holds all (default and non-default) parameter key-value
+        pairs passed to the frontend
+
+    """
+
+    # create new cfg dict
+    new_cfg = StructDict()
+    for par_name in defaults:
+        # check only needed for injected kwargs like `parallel`
+        if par_name in lcls:
+            new_cfg[par_name] = lcls[par_name]
+    # attach additional kwargs (like select)
+    for key in kwargs:
+        new_cfg[key] = kwargs[key]
+
+    return new_cfg
+
+
 def best_match(source, selection, span=False, tol=None, squash_duplicates=False):
     """
     Find matching elements in a given 1d-array/list
@@ -147,7 +217,7 @@ def best_match(source, selection, span=False, tol=None, squash_duplicates=False)
                              np.where(source <= selection[1])[0])
     else:
         issorted = True
-        if np.diff(source).min() < 0:
+        if source.size > 1 and np.diff(source).min() < 0:
             issorted = False
             orig = np.array(source, copy=True)
             idx_orig = np.argsort(orig)
@@ -199,5 +269,3 @@ def get_defaults(obj):
     dct = {k: v.default for k, v in inspect.signature(obj).parameters.items()\
            if v.default != v.empty and v.name != "cfg"}
     return StructDict(dct)
-
-
