@@ -8,7 +8,7 @@ import numpy as np
 
 # Syncopy imports
 from syncopy.shared.parsers import data_parser, scalar_parser, array_parser
-from syncopy.shared.tools import get_defaults
+from syncopy.shared.tools import get_defaults, get_frontend_cfg
 from syncopy.datatype import SpectralData
 from syncopy.shared.errors import SPYValueError, SPYTypeError, SPYWarning, SPYInfo
 from syncopy.shared.kwarg_decorators import (unwrap_cfg, unwrap_select,
@@ -25,34 +25,37 @@ from syncopy.shared.input_processors import (
 )
 
 # method specific imports - they should go!
+from syncopy.specest.fooofspy import default_fooof_opt
 import syncopy.specest.wavelets as spywave
 import syncopy.specest.superlet as superlet
 from .wavelet import get_optimal_wavelet_scales
 
 # Local imports
-from .const_def import (
-    availableWavelets,
-    availableMethods,
-)
 
 from .compRoutines import (
     SuperletTransform,
     WaveletTransform,
     MultiTaperFFT,
-    MultiTaperFFTConvol
+    MultiTaperFFTConvol,
+    FooofSpy
 )
 
+availableFooofOutputs = ['fooof', 'fooof_aperiodic', 'fooof_peaks']
+availableOutputs = tuple(spectralConversions.keys())
+availableWavelets = ("Morlet", "Paul", "DOG", "Ricker", "Marr", "Mexican_hat")
+availableMethods = ("mtmfft", "mtmconvol", "wavelet", "superlet")
+
 
 @unwrap_cfg
 @unwrap_select
 @detect_parallel_client
 def freqanalysis(data, method='mtmfft', output='pow',
                  keeptrials=True, foi=None, foilim=None,
-                 pad_to_length=None, polyremoval=None, taper="hann",
+                 pad='maxperlen', polyremoval=0, taper="hann",
                  taper_opt=None, tapsmofrq=None, nTaper=None, keeptapers=False,
                  toi="all", t_ftimwin=None, wavelet="Morlet", width=6, order=None,
                  order_max=None, order_min=1, c_1=3, adaptive=False,
-                 out=None, **kwargs):
+                 out=None, fooof_opt=None, **kwargs):
     """
     Perform (time-)frequency analysis of Syncopy :class:`~syncopy.AnalogData` objects
 
@@ -60,7 +63,7 @@ def freqanalysis(data, method='mtmfft', output='pow',
 
     Options available in all analysis methods:
 
-    * **output** : one of :data:`~syncopy.specest.const_def.availableOutputs`;
+    * **output** : one of :data:`~syncopy.specest.freqanalysis.availableOutputs`;
       return power spectra, complex Fourier spectra or absolute values.
     * **foi**/**foilim** : frequencies of interest; either array of frequencies or
       frequency window (not both)
@@ -79,7 +82,16 @@ def freqanalysis(data, method='mtmfft', output='pow',
         * **tapsmofrq** : spectral smoothing box for slepian tapers (in Hz)
         * **nTaper** : number of orthogonal tapers for slepian tapers
         * **keeptapers** : return individual tapers or average
-        * **pad_to_length**: either pad to an absolute length or set to `'nextpow2'`
+        * **pad**: either pad to an absolute length or set to `'nextpow2'`
+
+        Post-processing of the resulting spectra with FOOOOF is available
+        via setting `output` to one of `'fooof'`, `'fooof_aperiodic'` or
+        `'fooof_peaks'`, see below for details. The returned spectrum represents
+        the full foofed spectrum for `'fooof'`, the aperiodic
+        fit for `'fooof_aperiodic'`, and the peaks (Gaussians fit to them) for
+        `'fooof_peaks'`. Returned data is in linear scale. Noisy input
+        data will most likely lead to fitting issues with fooof, always inspect
+        your results!
 
     "mtmconvol" : (Multi-)tapered sliding window Fourier transform
         Perform time-frequency analysis on time-series trial data based on a sliding
@@ -100,7 +112,7 @@ def freqanalysis(data, method='mtmfft', output='pow',
         Perform time-frequency analysis on time-series trial data using a non-orthogonal
         continuous wavelet transform.
 
-        * **wavelet** : one of :data:`~syncopy.specest.const_def.availableWavelets`
+        * **wavelet** : one of :data:`~syncopy.specest.freqanalysis.availableWavelets`
         * **toi** : time-points of interest; can be either an array representing
           time points (in sec) or "all"(pre-trimming and subsampling of results)
         * **width** : Nondimensional frequency constant of Morlet wavelet function (>= 6)
@@ -124,13 +136,15 @@ def freqanalysis(data, method='mtmfft', output='pow',
     data : `~syncopy.AnalogData`
         A non-empty Syncopy :class:`~syncopy.datatype.AnalogData` object
     method : str
-        Spectral estimation method, one of :data:`~syncopy.specest.const_def.availableMethods`
+        Spectral estimation method, one of :data:`~syncopy.specest.freqanalysis.availableMethods`
         (see below).
     output : str
-        Output of spectral estimation. One of :data:`~syncopy.specest.const_def.availableOutputs` (see below);
+        Output of spectral estimation. One of :data:`~syncopy.specest.freqanalysis.availableOutputs` (see below);
         use `'pow'` for power spectrum (:obj:`numpy.float32`), `'fourier'` for complex
         Fourier coefficients (:obj:`numpy.complex64`) or `'abs'` for absolute
-        values (:obj:`numpy.float32`).
+        values (:obj:`numpy.float32`). Use one of `'fooof'`, `'fooof_aperiodic'` or
+        `'fooof_peaks'` to request post-processing of the results with FOOOF, also see
+        the `'fooof_opt'` parameter description.
     keeptrials : bool
         If `True` spectral estimates of individual trials are returned, otherwise
         results are averaged across trials.
@@ -145,15 +159,15 @@ def freqanalysis(data, method='mtmfft', output='pow',
         but may be unbounded (e.g., ``[-np.inf, 60.5]`` is valid). Edges `fmin`
         and `fmax` are included in the selection. If `foilim` is `None` or
         ``foilim = "all"``, all frequencies are selected.
-    pad_to_length : int, None or 'nextpow2'
-        Padding of the input data, if set to a number pads all trials
-        to this absolute length. For instance ``pad_to_length = 2000`` pads all
-        trials to an absolute length of 2000 samples, if and only if the longest
-        trial contains at maximum 2000 samples.
-        Alternatively `pad_to_length='nextpow2'` pads all trials to
-        the next power of two of the longest trial.
-        If `None` and trials have unequal lengths all trials are padded to match
-        the longest trial.
+    pad : 'maxperlen', float or 'nextpow2'
+        For the default `maxperlen`, no padding is performed in case of equal
+        length trials, while trials of varying lengths are padded to match the
+        longest trial. If `pad` is a number all trials are padded so that `pad` indicates
+        the absolute length of all trials after padding (in seconds). For instance
+        ``pad = 2`` pads all trials to an absolute length of 2000 samples, if and
+        only if the longest trial contains at maximum 2000 samples and the
+        samplerate is 1kHz. If `pad` is `'nextpow2'` all trials are padded to the
+        nearest power of two (in samples) of the longest trial.
     polyremoval : int or None
         Order of polynomial used for de-trending data in the time domain prior
         to spectral analysis. A value of 0 corresponds to subtracting the mean
@@ -164,7 +178,7 @@ def freqanalysis(data, method='mtmfft', output='pow',
         default.
     tapsmofrq : float or None
         Only valid if `method` is `'mtmfft'` or `'mtmconvol'`
-        Enables multi-tapering and sets the amount of spectral
+        Enables multi-tapering and sets the amount of one-sided spectral
         smoothing with slepian tapers in Hz.
     nTaper : int or None
         Only valid if `method` is `'mtmfft'` or `'mtmconvol'` and `tapsmofrq` is set.
@@ -172,7 +186,7 @@ def freqanalysis(data, method='mtmfft', output='pow',
         of tapers manually! Leave at `None` for the optimal number to be set automatically.
     taper : str or None, optional
         Only valid if `method` is `'mtmfft'` or `'mtmconvol'`. Windowing function,
-        one of :data:`~syncopy.specest.const_def.availableTapers`
+        one of :data:`~syncopy.shared.const_def.availableTapers`
         For multi-tapering with slepian tapers use `tapsmofrq` directly.
     taper_opt : dict or None
         Dictionary with keys for additional taper parameters.
@@ -199,7 +213,7 @@ def freqanalysis(data, method='mtmfft', output='pow',
         Only valid if `method` is `'mtmconvol'`. Sliding window length (in seconds).
     wavelet : str
         Only valid if `method` is `'wavelet'`. Wavelet function to use, one of
-        :data:`~syncopy.specest.const_def.availableWavelets` (see below).
+        :data:`~syncopy.specest.freqanalysis.availableWavelets` (see below).
     width : positive float
         Only valid if `method` is `'wavelet'` and `wavelet` is `'Morlet'`. Nondimensional
         frequency constant of Morlet wavelet function. This number should be >= 6,
@@ -241,8 +255,17 @@ def freqanalysis(data, method='mtmfft', output='pow',
         linearly with the frequencies of interest from `order_min`
         to `order_max`. If set to False the same SL will be used for
         all frequencies.
-    out : None or :class:`SpectralData` object
-        None if a new :class:`SpectralData` object is to be created, or an empty :class:`SpectralData` object
+    fooof_opt : dict or None
+        Only valid if `method` is `'mtmfft'` and `output` is  one of
+        `'fooof'`, `'fooof_aperiodic'`, or `'fooof_peaks'`.
+        Additional keyword arguments passed to the `FOOOF` constructor. Available
+        arguments include ``'peak_width_limits'``, ``'max_n_peaks'``, ``'min_peak_height'``,
+        ``'peak_threshold'``, and ``'aperiodic_mode'``.
+        Please refer to the
+        `FOOOF docs <https://fooof-tools.github.io/fooof/generated/fooof.FOOOF.html#fooof.FOOOF>`_
+        for the meanings and the defaults.
+        The FOOOF reference is: Donoghue et al. 2020, DOI 10.1038/s41593-020-00744-x.
+    out : Must be `None`.
 
 
     Returns
@@ -257,13 +280,13 @@ def freqanalysis(data, method='mtmfft', output='pow',
 
     **Options**
 
-    .. autodata:: syncopy.specest.const_def.availableMethods
+    .. autodata:: syncopy.specest.freqanalysis.availableMethods
 
-    .. autodata:: syncopy.specest.const_def.availableOutputs
+    .. autodata:: syncopy.specest.freqanalysis.availableOutputs
 
     .. autodata:: syncopy.shared.const_def.availableTapers
 
-    .. autodata:: syncopy.specest.const_def.availableWavelets
+    .. autodata:: syncopy.specest.freqanalysis.availableWavelets
 
     Examples
     --------
@@ -276,6 +299,7 @@ def freqanalysis(data, method='mtmfft', output='pow',
     syncopy.specest.mtmfft.mtmfft : (multi-)tapered Fourier transform of multi-channel time series data
     syncopy.specest.mtmconvol.mtmconvol : time-frequency analysis of multi-channel time series data with a sliding window FFT
     syncopy.specest.wavelet.wavelet : time-frequency analysis of multi-channel time series data using a wavelet transform
+    syncopy.specest.fooofspy.fooofspy : parameterization of neural power spectra with the 'fitting oscillations & one over f' method
     numpy.fft.fft : NumPy's reference FFT implementation
     scipy.signal.stft : SciPy's Short Time Fourier Transform
     """
@@ -290,18 +314,33 @@ def freqanalysis(data, method='mtmfft', output='pow',
 
     # Get everything of interest in local namespace
     defaults = get_defaults(freqanalysis)
+
     lcls = locals()
     # check for ineffective additional kwargs
     check_passed_kwargs(lcls, defaults, frontend_name="freqanalysis")
 
+    new_cfg = get_frontend_cfg(defaults, lcls, kwargs)
+
+    is_fooof = False
+    if method == "mtmfft" and output.startswith("fooof"):
+        is_fooof = True
+        output_fooof = output
+        output = "pow"  # We need to change this as the mtmfft running first will complain otherwise.
+
+    if is_fooof:
+        if output_fooof not in availableFooofOutputs:
+            lgl = "'" + "or '".join(opt + "' " for opt in availableFooofOutputs)
+            raise SPYValueError(legal=lgl, varname="output_fooof", actual=output_fooof)
+
     # Ensure a valid computational method was selected
     if method not in availableMethods:
         lgl = "'" + "or '".join(opt + "' " for opt in availableMethods)
         raise SPYValueError(legal=lgl, varname="method", actual=method)
 
     # Ensure a valid output format was selected
-    if output not in spectralConversions.keys():
-        lgl = "'" + "or '".join(opt + "' " for opt in spectralConversions.keys())
+    valid_outputs = spectralConversions.keys()
+    if output not in valid_outputs:
+        lgl = "'" + "or '".join(opt + "' " for opt in valid_outputs)
         raise SPYValueError(legal=lgl, varname="output", actual=output)
 
     # Parse all Boolean keyword arguments
@@ -312,6 +351,10 @@ def freqanalysis(data, method='mtmfft', output='pow',
     # If only a subset of `data` is to be processed, make some necessary adjustments
     # of the sampleinfo and trial lengths
     if data.selection is not None:
+        # Refuse to go ahead with active time selection and provided `toi` on top`
+        if any(tsel != slice(None) for tsel in data.selection.time) and isinstance(toi, (np.ndarray, list)):
+            lgl = "no `toi` specification due to active in-place time-selection in input dataset"
+            raise SPYValueError(legal=lgl, varname="toi", actual=toi)
         sinfo = data.selection.trialdefinition[:, :2]
         trialList = data.selection.trials
     else:
@@ -326,22 +369,21 @@ def freqanalysis(data, method='mtmfft', output='pow',
     if polyremoval is not None:
         scalar_parser(polyremoval, varname="polyremoval", ntype="int_like", lims=[0, 1])
 
-
     # --- Padding ---
 
     # Sliding window FFT does not support "fancy" padding
-    if method == "mtmconvol" and isinstance(pad_to_length, str):
+    if method == "mtmconvol" and isinstance(pad, str):
         msg = "method 'mtmconvol' only supports in-place padding for windows " +\
-            "exceeding trial boundaries. Your choice of `pad_to_length = '{}'` will be ignored. "
-        SPYWarning(msg.format(pad_to_length))
+            "exceeding trial boundaries. Your choice of `pad = '{}'` will be ignored. "
+        SPYWarning(msg.format(pad))
 
     if method == 'mtmfft':
         # the actual number of samples in case of later padding
-        minSampleNum = process_padding(pad_to_length, lenTrials)
+        minSampleNum = process_padding(pad, lenTrials, data.samplerate)
     else:
         minSampleNum = lenTrials.min()
 
-    # Compute length (in samples) of shortest trial
+    # Compute length (in seconds) of shortest trial
     minTrialLength = minSampleNum / data.samplerate
 
     # Shortcut to data sampling interval
@@ -359,11 +401,11 @@ def freqanalysis(data, method='mtmfft', output='pow',
     # Prepare keyword dict for logging (use `lcls` to get actually provided
     # keyword values, not defaults set above)
     log_dct = {"method": method,
-               "output": output,
+               "output": output_fooof if is_fooof else output,
                "keeptapers": keeptapers,
                "keeptrials": keeptrials,
                "polyremoval": polyremoval,
-               "pad_to_length": pad_to_length}
+               "pad": pad}
 
     # --------------------------------
     # 1st: Check time-frequency inputs
@@ -429,7 +471,6 @@ def freqanalysis(data, method='mtmfft', output='pow',
             lgl = "array of equidistant time-points or 'all' for wavelet based methods"
             raise SPYValueError(legal=lgl, varname="toi", actual=toi)
 
-
         # Update `log_dct` w/method-specific options (use `lcls` to get actually
         # provided keyword values, not defaults set in here)
         log_dct["toi"] = lcls["toi"]
@@ -584,9 +625,9 @@ def freqanalysis(data, method='mtmfft', output='pow',
         # number of samples per window
         nperseg = int(t_ftimwin * data.samplerate)
         halfWin = int(nperseg / 2)
-        postSelect = slice(None) # select all is the default
+        postSelect = slice(None)  # select all is the default
 
-        if 0 <= overlap <= 1: # `toi` is percentage
+        if 0 <= overlap <= 1:  # `toi` is percentage
             noverlap = min(nperseg - 1, int(overlap * nperseg))
         # windows get shifted exactly 1 sample
         # to get a spectral estimate at each sample
@@ -603,6 +644,11 @@ def freqanalysis(data, method='mtmfft', output='pow',
             padEnd = halfWin - offEnd
             padEnd = ((padEnd > 0) * padEnd).astype(np.intp)
 
+            # Treat equi-distant `toi` arrays with spacing large enough that windows
+            # do not overlap as if they were not equidistant
+            if tSteps.max() * data.samplerate > halfWin and equidistant:
+                equidistant = False
+
             # Compute sample-indices (one slice/list per trial) from time-selections
             soi = []
             if equidistant:
@@ -702,7 +748,7 @@ def freqanalysis(data, method='mtmfft', output='pow',
         # automatic frequency selection
         if foi is None and foilim is None:
             scales = get_optimal_wavelet_scales(
-                wfun.scale_from_period, # all availableWavelets sport one!
+                wfun.scale_from_period,  # all availableWavelets sport one!
                 int(minTrialLength * data.samplerate),
                 dt)
             foi = 1 / wfun.fourier_period(scales)
@@ -725,9 +771,9 @@ def freqanalysis(data, method='mtmfft', output='pow',
 
         # method specific parameters
         method_kwargs = {
-            'samplerate' : data.samplerate,
-            'scales' : scales,
-            'wavelet' : wfun
+            'samplerate': data.samplerate,
+            'scales': scales,
+            'wavelet': wfun
         }
 
         # Set up compute-class
@@ -801,12 +847,12 @@ def freqanalysis(data, method='mtmfft', output='pow',
 
         # method specific parameters
         method_kwargs = {
-            'samplerate' : data.samplerate,
-            'scales' : scales,
-            'order_max' : order_max,
-            'order_min' : order_min,
-            'c_1' : c_1,
-            'adaptive' : adaptive
+            'samplerate': data.samplerate,
+            'scales': scales,
+            'order_max': order_max,
+            'order_min': order_min,
+            'c_1': c_1,
+            'adaptive': adaptive
         }
 
         # Set up compute-class
@@ -823,18 +869,7 @@ def freqanalysis(data, method='mtmfft', output='pow',
     # Sanitize output and call the ComputationalRoutine
     # -------------------------------------------------
 
-    # If provided, make sure output object is appropriate
-    if out is not None:
-        try:
-            data_parser(out, varname="out", writable=True, empty=True,
-                        dataclass="SpectralData",
-                        dimord=SpectralData().dimord)
-        except Exception as exc:
-            raise exc
-        new_out = False
-    else:
-        out = SpectralData(dimord=SpectralData._defaultDimord)
-        new_out = True
+    out = SpectralData(dimord=SpectralData._defaultDimord)
 
     # Perform actual computation
     specestMethod.initialize(data,
@@ -843,5 +878,57 @@ def freqanalysis(data, method='mtmfft', output='pow',
                              keeptrials=keeptrials)
     specestMethod.compute(data, out, parallel=kwargs.get("parallel"), log_dict=log_dct)
 
-    # Either return newly created output object or simply quit
-    return out if new_out else None
+    # FOOOF is a post-processing method of MTMFFT output, so we handle it here, once
+    # the MTMFFT has finished.
+    if is_fooof:
+
+        # Use the output of the MTMFFMT method as the new data and create new output data.
+        fooof_data = out
+        fooof_out = SpectralData(dimord=SpectralData._defaultDimord)
+
+        # method specific parameters
+        if fooof_opt is None:
+            fooof_opt = default_fooof_opt
+
+        # These go into the FOOOF constructor, so we keep them separate from the fooof_settings below.
+        fooof_kwargs = {**default_fooof_opt, **fooof_opt}  # Join the ones from fooof_opt (the user) into fooof_kwargs.
+
+        # Settings used during the FOOOF analysis (that are NOT passed to FOOOF constructor).
+        # The user cannot influence these: in_freqs is derived from mtmfft output, freq_range is always None (=full mtmfft output spectrum).
+        # We still define them here, and they are passed through to the backend and actually used there.
+        fooof_settings = {
+            'in_freqs': fooof_data.freq,
+            'freq_range': None  # or something like [2, 40] to limit frequency range (post processing). Currently not exposed to user.
+        }
+
+        if fooof_data.freq[0] == 0:
+            # FOOOF does not work with input frequency zero in the data.
+            raise SPYValueError(legal="a frequency range that does not include zero. Use 'foi' or 'foilim' to restrict.", varname="foi/foilim", actual="Frequency range from {} to {}.".format(min(fooof_data.freq), max(fooof_data.freq)))
+
+        # Set up compute-class
+        #  - the output_fmt must be one of 'fooof', 'fooof_aperiodic',
+        #    or 'fooof_peaks'.
+        #  - everything passed as method_kwargs is passed as arguments
+        #    to the foooof.FOOOF() constructor or functions, the other args are
+        #    used elsewhere.
+        fooofMethod = FooofSpy(output_fmt=output_fooof, fooof_settings=fooof_settings, method_kwargs=fooof_kwargs)
+
+        # Update `log_dct` w/method-specific options
+        log_dct["fooof_method"] = output_fooof
+        log_dct["fooof_opt"] = fooof_kwargs
+
+        # Perform actual computation
+        fooofMethod.initialize(fooof_data,
+                               fooof_out._stackingDim,
+                               chan_per_worker=kwargs.get("chan_per_worker"),
+                               keeptrials=keeptrials)
+        fooofMethod.compute(fooof_data, fooof_out, parallel=kwargs.get("parallel"), log_dict=log_dct)
+        out = fooof_out
+
+     # attach potential older cfg's from the input
+    # to support chained frontend calls..
+    out.cfg.update(data.cfg)
+
+    # attach frontend parameters for replay
+    out.cfg.update({'freqanalysis': new_cfg})
+    return out
