@@ -20,14 +20,14 @@ from syncopy.datatype import AnalogData
 from syncopy.datatype.base_data import Selector
 from syncopy.io import load
 from syncopy.shared.computational_routine import ComputationalRoutine
-from syncopy.shared.kwarg_decorators import unwrap_io, unwrap_cfg, unwrap_select
+from syncopy.shared.kwarg_decorators import process_io, unwrap_cfg, unwrap_select
 from syncopy.tests.misc import generate_artificial_data
 
 # Decorator to decide whether or not to run dask-related tests
 skip_without_acme = pytest.mark.skipif(not __acme__, reason="acme not available")
 
 
-@unwrap_io
+@process_io
 def lowpass(arr, b, a=None, noCompute=None, chunkShape=None):
     if noCompute:
         return arr.shape, arr.dtype
@@ -235,9 +235,6 @@ class TestComputationalRoutine():
             out = filter_manager(self.sigdata, self.b, self.a, select=select,
                                  log_dict={"a": "this is a", "b": "this is b"})
 
-            # only keyword args (`a` in this case here) are stored in `cfg`
-            assert set(["a"]) == set(out.cfg.keys())
-            assert np.array_equal(out.cfg["a"], self.a)
             assert len(out.trials) == len(sel.trials)
             # ensure our `log_dict` specification was respected
             assert "lowpass" in out._log
@@ -251,8 +248,6 @@ class TestComputationalRoutine():
                 selected = self.sigdata.selectdata(**select)
             out_sel = filter_manager(selected, self.b, self.a,
                                      log_dict={"a": "this is a", "b": "this is b"})
-            assert set(["a"]) == set(out.cfg.keys())
-            assert np.array_equal(out.cfg["a"], self.a)
             assert len(out.trials) == len(out_sel.trials)
             assert "lowpass" in out._log
             assert "a = this is a" in out._log
@@ -263,8 +258,6 @@ class TestComputationalRoutine():
                 fname = os.path.join(tdir, "dummy")
                 out.save(fname)
                 dummy = load(fname)
-                assert "a" in dummy.cfg.keys()
-                assert np.array_equal(dummy.cfg["a"], self.a)
                 assert out.filename == dummy.filename
                 if select is None:
                     reference = self.orig
@@ -283,8 +276,6 @@ class TestComputationalRoutine():
                 fname2 = os.path.join(tdir, "dummy2")
                 out_sel.save(fname2)
                 dummy2 = load(fname2)
-                assert "a" in dummy2.cfg.keys()
-                assert np.array_equal(dummy2.cfg["a"], dummy.cfg["a"])
                 assert np.array_equal(dummy.data, dummy2.data)
                 assert np.array_equal(dummy.channel, dummy2.channel)
                 assert np.array_equal(dummy.time, dummy2.time)
@@ -440,9 +431,6 @@ class TestComputationalRoutine():
                                      log_dict={"a": "this is a", "b": "this is b"},
                                      parallel=True, parallel_store=parallel_store)
 
-                # only keyword args (`a` in this case here) are stored in `cfg`
-                assert set(["a"]) == set(out.cfg.keys())
-                assert np.array_equal(out.cfg["a"], self.a)
                 assert len(out.trials) == len(sel.trials)
                 # ensure our `log_dict` specification was respected
                 assert "lowpass" in out._log
@@ -458,8 +446,6 @@ class TestComputationalRoutine():
                                          log_dict={"a": "this is a", "b": "this is b"},
                                          parallel=True, parallel_store=parallel_store)
                 # only keyword args (`a` in this case here) are stored in `cfg`
-                assert set(["a"]) == set(out.cfg.keys())
-                assert np.array_equal(out.cfg["a"], self.a)
                 assert len(out.trials) == len(sel.trials)
                 # ensure our `log_dict` specification was respected
                 assert "lowpass" in out._log
@@ -471,8 +457,6 @@ class TestComputationalRoutine():
                     fname = os.path.join(tdir, "dummy")
                     out.save(fname)
                     dummy = load(fname)
-                    assert "a" in dummy.cfg.keys()
-                    assert np.array_equal(dummy.cfg["a"], self.a)
                     assert out.filename == dummy.filename
                     assert not out.data.is_virtual
                     if select is None:
@@ -491,8 +475,6 @@ class TestComputationalRoutine():
                     fname2 = os.path.join(tdir, "dummy2")
                     out_sel.save(fname2)
                     dummy2 = load(fname2)
-                    assert "a" in dummy2.cfg.keys()
-                    assert np.array_equal(dummy2.cfg["a"], dummy.cfg["a"])
                     assert np.array_equal(dummy.data, dummy2.data)
                     assert np.array_equal(dummy.channel, dummy2.channel)
                     assert np.array_equal(dummy.time, dummy2.time)
