@@ -59,6 +59,11 @@ class TestSpikeData():
         with pytest.raises(SPYTypeError):
             SpikeData({})
 
+    def test_issue_257_fixed_no_error_for_empty_data(self):
+        """This tests that the data object is created without throwing an error, see #257."""
+        data = SpikeData(np.column_stack(([],[],[])), dimord = ['sample', 'channel', 'unit'], samplerate = 30000)
+        assert data.dimord == ["sample", "channel", "unit"]
+
     def test_nparray(self):
         dummy = SpikeData(self.data)
         assert dummy.dimord == ["sample", "channel", "unit"]
@@ -170,21 +175,21 @@ class TestSpikeData():
             [4, 2, 2, 5, 5],   # repetition + unorderd
             range(5, 8),  # narrow range
             slice(-5, None)  # negative-start slice
-            ]
+        ]
         toiSelections = [
             "all",  # non-type-conform string
             [-0.2, 0.6, 0.9, 1.1, 1.3, 1.6, 1.8, 2.2, 2.45, 3.]  # unordered, inexact, repetions
-            ]
+        ]
         toilimSelections = [
             [0.5, 3.5],  # regular range
             [1.0, np.inf]  # unbounded from above
-            ]
+        ]
         unitSelections = [
             ["unit1", "unit1", "unit2", "unit3"],  # preserve repetition
             [0, 0, 2, 3],  # preserve repetition, don't convert to slice
             range(1, 4),  # narrow range
             slice(-2, None)  # negative-start slice
-            ]
+        ]
         timeSelections = list(zip(["toi"] * len(toiSelections), toiSelections)) \
             + list(zip(["toilim"] * len(toilimSelections), toilimSelections))
 
@@ -221,19 +226,18 @@ class TestSpikeData():
                             for trialno in selector.trials:
                                 if selector.time[tk]:
                                     assert np.array_equal(obj.trials[trialno][selector.time[tk], :],
-                                                        selected.trials[tk])
+                                                          selected.trials[tk])
                                     tk += 1
                             assert set(selected.data[:, chanIdx]).issubset(chanArr[selector.channel])
                             assert set(selected.channel) == set(obj.channel[selector.channel])
                             assert np.array_equal(selected.unit,
-                                                obj.unit[np.unique(selected.data[:, unitIdx])])
+                                                  obj.unit[np.unique(selected.data[:, unitIdx])])
                             cfg.data = obj
-                            cfg.out = SpikeData(dimord=obj.dimord)
                             # data selection via package function and `cfg`: ensure equality
-                            selectdata(cfg)
-                            assert np.array_equal(cfg.out.channel, selected.channel)
-                            assert np.array_equal(cfg.out.unit, selected.unit)
-                            assert np.array_equal(cfg.out.data, selected.data)
+                            out = selectdata(cfg)
+                            assert np.array_equal(out.channel, selected.channel)
+                            assert np.array_equal(out.unit, selected.unit)
+                            assert np.array_equal(out.data, selected.data)
 
     @skip_without_acme
     def test_parallel(self, testcluster, fulltests):
@@ -245,6 +249,7 @@ class TestSpikeData():
             flush_local_cluster(testcluster)
         client.close()
 
+
 class TestEventData():
 
     # Allocate test-datasets
@@ -256,19 +261,21 @@ class TestEventData():
     data2 = data.copy()
     data2[:, -1] = data[:, 0]
     data2[:, 0] = data[:, -1]
+    data3 = np.hstack([data2, data2])
     trl = np.vstack([np.arange(0, ns, 5),
                      np.arange(5, ns + 5, 5),
                      np.ones((int(ns / 5), )),
                      np.ones((int(ns / 5), )) * np.pi]).T
     num_smp = np.unique(data[:, 0]).size
     num_evt = np.unique(data[:, 1]).size
+    customDimord = ["sample", "eventid", "custom1", "custom2"]
 
     adata = np.arange(1, nc * ns + 1).reshape(ns, nc)
 
     def test_ed_empty(self):
         dummy = EventData()
         assert len(dummy.cfg) == 0
-        assert dummy.dimord == None
+        assert dummy.dimord is None
         for attr in ["data", "sampleinfo", "samplerate", "trialid", "trialinfo"]:
             assert getattr(dummy, attr) is None
         with pytest.raises(SPYTypeError):
@@ -298,7 +305,7 @@ class TestEventData():
             trl_ref = self.data[idx, ...]
             assert np.array_equal(dummy._get_trial(trlno), trl_ref)
 
-        # test ``_get_trial`` with NumPy array: swapped dimensions
+        # test `_get_trial` with NumPy array: swapped dimensions
         dummy = EventData(self.data2, trialdefinition=self.trl,
                           dimord=["eventid", "sample"])
         smp = self.data2[:, -1]
@@ -308,6 +315,18 @@ class TestEventData():
             trl_ref = self.data2[idx, ...]
             assert np.array_equal(dummy._get_trial(trlno), trl_ref)
 
+        # test `_get_trial` with NumPy array: customized columns names
+        nuDimord = ["eventid", "sample", "custom1", "custom2"]
+        dummy = EventData(self.data3, trialdefinition=self.trl,
+                          dimord=nuDimord)
+        assert dummy.dimord == nuDimord
+        smp = self.data3[:, -1]
+        for trlno, start in enumerate(range(0, self.ns, 5)):
+            idx = np.intersect1d(np.where(smp >= start)[0],
+                                 np.where(smp < start + 5)[0])
+            trl_ref = self.data3[idx, ...]
+            assert np.array_equal(dummy._get_trial(trlno), trl_ref)
+
     def test_ed_saveload(self):
         with tempfile.TemporaryDirectory() as tdir:
             fname = os.path.join(tdir, "dummy")
@@ -352,7 +371,17 @@ class TestEventData():
             filename = construct_spy_filename(fname + "_dimswap", dummy)
             dummy2 = load(filename)
             assert dummy2.dimord == dummy.dimord
-            assert dummy2.eventid.size == self.num_smp # swapped
+            assert dummy2.eventid.size == self.num_smp  # swapped
+            assert dummy2.data.shape == dummy.data.shape
+            del dummy, dummy2
+
+            # save dataset w/custom column names and ensure `dimord` is preserved
+            dummy = EventData(np.hstack([self.data, self.data]), dimord=self.customDimord, samplerate=10)
+            dummy.save(fname + "_customDimord")
+            filename = construct_spy_filename(fname + "_customDimord", dummy)
+            dummy2 = load(filename)
+            assert dummy2.dimord == dummy.dimord
+            assert dummy2.eventid.size == self.num_evt
             assert dummy2.data.shape == dummy.data.shape
 
             # Delete all open references to file objects b4 closing tmp dir
@@ -388,9 +417,10 @@ class TestEventData():
         samples = np.arange(0, int(self.ns / 3), 3)[1:]
         dappend = np.vstack([samples, np.full(samples.shape, 2)]).T
         data3 = np.vstack([self.data, dappend])
+        data3 = np.hstack([data3, data3])
         idx = np.argsort(data3[:, 0])
         data3 = data3[idx, :]
-        evt_dummy = EventData(data3, samplerate=sr_e)
+        evt_dummy = EventData(data3, dimord=self.customDimord, samplerate=sr_e)
         evt_dummy.definetrial(start=0, stop=1)
         assert np.array_equal(sinfo2, evt_dummy.sampleinfo)
 
@@ -410,7 +440,7 @@ class TestEventData():
             dcodes = dcodes[idx + 1:]
             dsamps = dsamps[idx + 1:]
             sinfo3[sk, :] = [start, stop]
-        evt_dummy = EventData(data3, samplerate=sr_e)
+        evt_dummy = EventData(data3, dimord=self.customDimord, samplerate=sr_e)
         evt_dummy.definetrial(start=[2, 2, 1], stop=[1, 2, 0])
         assert np.array_equal(evt_dummy.sampleinfo, sinfo3)
 
@@ -420,7 +450,7 @@ class TestEventData():
         ang_dummy = AnalogData(self.adata, samplerate=sr_a)
         ang_dummy.definetrial(evt_dummy)
         assert np.array_equal(ang_dummy.sampleinfo, sinfo_a)
-        evt_dummy = EventData(data=data3, samplerate=sr_e)
+        evt_dummy = EventData(data=data3, dimord=self.customDimord, samplerate=sr_e)
         evt_dummy.definetrial(pre=pre, post=post, trigger=1)
         ang_dummy.definetrial(evt_dummy)
         assert np.array_equal(ang_dummy.sampleinfo, sinfo_a)
@@ -430,20 +460,21 @@ class TestEventData():
         ang_dummy = AnalogData(self.adata, samplerate=sr_a)
         ang_dummy.definetrial(evt_dummy, pre=pre, post=post, trigger=1)
         assert np.array_equal(ang_dummy.sampleinfo, sinfo_a)
-        evt_dummy = EventData(data=data3, samplerate=sr_e)
+        evt_dummy = EventData(data=data3, dimord=self.customDimord, samplerate=sr_e)
         ang_dummy = AnalogData(self.adata, samplerate=sr_a)
         ang_dummy.definetrial(evt_dummy, pre=pre, post=post, trigger=1)
         assert np.array_equal(ang_dummy.sampleinfo, sinfo_a)
 
-        # Extend data and provoke an exception due to out of bounds erro
+        # Extend data and provoke an exception due to out of bounds error
         smp = np.vstack([np.arange(self.ns, int(2.5 * self.ns), 5),
                          np.zeros((int((1.5 * self.ns) / 5),))]).T
         smp[1::2, 1] = 1
+        smp = np.hstack([smp, smp])
         data4 = np.vstack([data3, smp])
-        evt_dummy = EventData(data=data4, samplerate=sr_e)
+        evt_dummy = EventData(data=data4, dimord=self.customDimord, samplerate=sr_e)
         evt_dummy.definetrial(pre=pre, post=post, trigger=1)
         # with pytest.raises(SPYValueError):
-            # ang_dummy.definetrial(evt_dummy)
+        # ang_dummy.definetrial(evt_dummy)
 
         # Trimming edges produces zero-length trial
         with pytest.raises(SPYValueError):
@@ -452,10 +483,10 @@ class TestEventData():
         # We need `clip_edges` to make trial-definition work
         data4 = data4[:-2, :]
         data4[-2, 0] = data4[-1, 0]
-        evt_dummy = EventData(data=data4, samplerate=sr_e)
+        evt_dummy = EventData(data=data4, dimord=self.customDimord, samplerate=sr_e)
         evt_dummy.definetrial(pre=pre, post=post, trigger=1)
         # with pytest.raises(SPYValueError):
-            # ang_dummy.definetrial(evt_dummy)
+        # ang_dummy.definetrial(evt_dummy)
         ang_dummy.definetrial(evt_dummy, clip_edges=True)
         assert ang_dummy.sampleinfo[-1, 1] == self.ns
 
@@ -495,10 +526,11 @@ class TestEventData():
     def test_ed_dataselection(self, fulltests):
 
         # Create testing objects (regular and swapped dimords)
-        dummy = EventData(data=self.data,
+        dummy = EventData(data=np.hstack([self.data, self.data]),
+                          dimord=self.customDimord,
                           trialdefinition=self.trl,
                           samplerate=2.0)
-        ymmud = EventData(data=self.data[:, ::-1],
+        ymmud = EventData(data=np.hstack([self.data[:, ::-1], self.data[:, ::-1]]),
                           trialdefinition=self.trl,
                           samplerate=2.0,
                           dimord=dummy.dimord[::-1])
@@ -512,15 +544,15 @@ class TestEventData():
             [0, 0, 1],  # preserve repetition, don't convert to slice
             range(0, 2),  # narrow range
             slice(-2, None)  # negative-start slice
-            ]
+        ]
         toiSelections = [
             "all",  # non-type-conform string
             [-0.2, 0.6, 0.9, 1.1, 1.3, 1.6, 1.8, 2.2, 2.45, 3.]  # unordered, inexact, repetions
-            ]
+        ]
         toilimSelections = [
             [0.5, 3.5],  # regular range
             [0.0, np.inf]  # unbounded from above
-            ]
+        ]
         timeSelections = list(zip(["toi"] * len(toiSelections), toiSelections)) \
             + list(zip(["toilim"] * len(toilimSelections), toilimSelections))
 
@@ -551,16 +583,15 @@ class TestEventData():
                         for trialno in selector.trials:
                             if selector.time[tk]:
                                 assert np.array_equal(obj.trials[trialno][selector.time[tk], :],
-                                                    selected.trials[tk])
+                                                      selected.trials[tk])
                                 tk += 1
                         assert np.array_equal(selected.eventid,
-                                            obj.eventid[np.unique(selected.data[:, eventidIdx]).astype(np.intp)])
+                                              obj.eventid[np.unique(selected.data[:, eventidIdx]).astype(np.intp)])
                         cfg.data = obj
-                        cfg.out = EventData(dimord=obj.dimord)
                         # data selection via package function and `cfg`: ensure equality
-                        selectdata(cfg)
-                        assert np.array_equal(cfg.out.eventid, selected.eventid)
-                        assert np.array_equal(cfg.out.data, selected.data)
+                        out = selectdata(cfg)
+                        assert np.array_equal(out.eventid, selected.eventid)
+                        assert np.array_equal(out.data, selected.data)
 
     @skip_without_acme
     def test_ed_parallel(self, testcluster, fulltests):
