@@ -11,13 +11,11 @@ import h5py
 import time
 import pytest
 import numpy as np
-from numpy.lib.format import open_memmap
 from glob import glob
-from memory_profiler import memory_usage
 
 # Local imports
 from syncopy.datatype import AnalogData
-from syncopy.io import save, load, load_ft_raw
+from syncopy.io import save, load, load_ft_raw, load_tdt
 from syncopy.shared.filetypes import FILE_EXT
 from syncopy.shared.errors import (
     SPYValueError,
@@ -30,7 +28,7 @@ from syncopy.tests.misc import generate_artificial_data
 
 
 # Decorator to detect if test data dir is available
-on_esi = os.path.isdir('/cs/scratch/syncopy')
+on_esi = os.path.isdir('/cs/slurm/syncopy')
 skip_no_esi = pytest.mark.skipif(not on_esi, reason="ESI fs not available")
 
 
@@ -46,37 +44,37 @@ class TestSpyIO():
     trl = {}
 
     # Generate 2D array simulating an AnalogData array
-    data["AnalogData"] = np.arange(1, nc*ns + 1).reshape(ns, nc)
+    data["AnalogData"] = np.arange(1, nc * ns + 1).reshape(ns, nc)
     trl["AnalogData"] = np.vstack([np.arange(0, ns, 5),
                                    np.arange(5, ns + 5, 5),
-                                   np.ones((int(ns/5), )),
-                                   np.ones((int(ns/5), )) * np.pi]).T
+                                   np.ones((int(ns / 5), )),
+                                   np.ones((int(ns / 5), )) * np.pi]).T
 
     # Generate a 4D array simulating a SpectralData array
-    data["SpectralData"] = np.arange(1, nc*ns*nt*nf + 1).reshape(ns, nt, nc, nf)
+    data["SpectralData"] = np.arange(1, nc * ns * nt * nf + 1).reshape(ns, nt, nc, nf)
     trl["SpectralData"] = trl["AnalogData"]
 
     # Generate a 4D array simulating a CorssSpectralData array
-    data["CrossSpectralData"] = np.arange(1, nc*nc*ns*nf + 1).reshape(ns, nf, nc, nc)
+    data["CrossSpectralData"] = np.arange(1, nc * nc * ns * nf + 1).reshape(ns, nf, nc, nc)
     trl["CrossSpectralData"] = trl["AnalogData"]
 
     # Use a fixed random number generator seed to simulate a 2D SpikeData array
     seed = np.random.RandomState(13)
     data["SpikeData"] = np.vstack([seed.choice(ns, size=nd),
                                    seed.choice(nc, size=nd),
-                                   seed.choice(int(nc/2), size=nd)]).T
+                                   seed.choice(int(nc / 2), size=nd)]).T
     trl["SpikeData"] = trl["AnalogData"]
 
     # Generate bogus trigger timings
     data["EventData"] = np.vstack([np.arange(0, ns, 5),
-                                   np.zeros((int(ns/5), ))]).T
+                                   np.zeros((int(ns / 5), ))]).T
     data["EventData"][1::2, 1] = 1
     trl["EventData"] = trl["AnalogData"]
 
     # Define data classes to be used in tests below
     classes = ["AnalogData", "SpectralData", "CrossSpectralData", "SpikeData", "EventData"]
 
-    # Test correct handling of object log and cfg
+    # Test correct handling of object log
     def test_logging(self):
         with tempfile.TemporaryDirectory() as tdir:
             fname = os.path.join(tdir, "dummy")
@@ -88,18 +86,12 @@ class TestSpyIO():
             assert len(dummy._log) > ldum
             assert dummy.filename in dummy._log
             assert dummy.filename + FILE_EXT["info"] in dummy._log
-            assert dummy.cfg["method"] == "save"
-            assert dummy.filename in dummy.cfg["files"]
-            assert dummy.filename + FILE_EXT["info"] in dummy.cfg["files"]
 
             # ensure loading is logged correctly
             dummy2 = load(filename=fname + ".analog")
             assert len(dummy2._log) > len(dummy._log)
             assert dummy2.filename in dummy2._log
             assert dummy2.filename + FILE_EXT["info"] in dummy._log
-            assert dummy2.cfg.cfg["method"] == "load"
-            assert dummy2.filename in dummy2.cfg.cfg["files"]
-            assert dummy2.filename + FILE_EXT["info"] in dummy2.cfg.cfg["files"]
 
             # Delete all open references to file objects b4 closing tmp dir
             del dummy, dummy2
@@ -446,31 +438,13 @@ class TestSpyIO():
                 load(os.path.join(tdir, container), dataclass=["invalid", "stillinvalid"],
                      tag="2nd")
 
-    def test_save_mmap(self):
-        with tempfile.TemporaryDirectory() as tdir:
-            fname = os.path.join(tdir, "vdat.npy")
-            dname = os.path.join(tdir, "dummy")
-            vdata = np.ones((1000, 5000))  # ca. 38.2 MB
-            np.save(fname, vdata)
-            del vdata
-            dmap = open_memmap(fname)
-            adata = AnalogData(dmap, samplerate=10)
-
-            # Ensure memory consumption stays within provided bounds
-            mem = memory_usage()[0]
-            save(adata, filename=dname, memuse=60)
-            assert (mem - memory_usage()[0]) < 70
-
-            # Delete all open references to file objects b4 closing tmp dir
-            del dmap, adata
-
 
 @skip_no_esi
-class Test_FT_Importer:
+class TestFTImporter:
 
     """At the moment only ft_datatype_raw is supported"""
 
-    mat_file_dir = '/cs/scratch/syncopy/MAT-Files'
+    mat_file_dir = '/cs/slurm/syncopy/MAT-Files'
 
     def test_read_hdf(self):
         """Test MAT-File v73 reader, uses h5py"""
@@ -535,7 +509,62 @@ class Test_FT_Importer:
 
         # additional fields of Matlab structures
         # get attached to .info dict
-        # here nested structures are supported (but dis-encouraged)
+        # here nested structures are also now forbidden
         dct = load_ft_raw(fname, include_fields=('ch',))
         AData2 = dct['Data_K']
-        assert 'ch' in AData2.info
+        # sadly here it is actually nested
+        assert len(AData2.info) == 0
+
+
+@skip_no_esi
+class TestTDTImporter:
+
+    tdt_dir = '/cs/slurm/syncopy/Tdt_reader/session-25'
+    start_code, end_code = 23000, 30020
+
+    def test_load_tdt(self):
+
+        AData = load_tdt(self.tdt_dir)
+
+        assert isinstance(AData, AnalogData)
+        # check meta info parsing
+        assert len(AData.info.keys()) == 13
+        # that is apparently fixed
+        assert AData.dimord == ['time', 'channel']
+        assert len(AData.channel) == 9
+
+        # it's only one big trial here
+        assert AData.trials[0].shape == (3170560, 9)
+
+        # test median subtr
+        AData2 = load_tdt(self.tdt_dir, subtract_median=True)
+        assert np.allclose(np.median(AData2.data), 0)
+
+        # check that it wasn't 0 before
+        assert not np.allclose(np.median(AData.data), 0)
+
+        # test automatic trialdefinition
+        AData = load_tdt(self.tdt_dir, self.start_code, self.end_code)
+        assert len(AData.trials) == 659
+
+    def test_exceptions(self):
+
+        with pytest.raises(SPYIOError, match='Cannot read'):
+            load_tdt('non/existing/path')
+
+        with pytest.raises(SPYValueError, match='Invalid value of `start_code`'):
+            load_tdt(self.tdt_dir, start_code=None, end_code=self.end_code)
+
+        with pytest.raises(SPYValueError, match='Invalid value of `end_code`'):
+            load_tdt(self.tdt_dir, start_code=self.start_code, end_code=None)
+
+        with pytest.raises(SPYValueError, match='Invalid value of `start_code`'):
+            load_tdt(self.tdt_dir, start_code=999999, end_code=self.end_code)
+
+        with pytest.raises(SPYValueError, match='Invalid value of `end_code`'):
+            load_tdt(self.tdt_dir, start_code=self.start_code, end_code=999999)
+
+
+if __name__ == '__main__':
+    T1 = TestFTImporter()
+    T2 = TestTDTImporter()
